import unittest

class TestServerSession(unittest.TestCase):
    def test___eq__(self):
        # server_session = ServerSession()
        # self.assertEqual(expected, server_session.__eq__(other))
        assert True # TODO: implement your test here

    def test___init__(self):
        # server_session = ServerSession()
        assert True # TODO: implement your test here

    def test___str__(self):
        # server_session = ServerSession()
        # self.assertEqual(expected, server_session.__str__())
        assert True # TODO: implement your test here

    def test___unicode__(self):
        # server_session = ServerSession()
        # self.assertEqual(expected, server_session.__unicode__())
        assert True # TODO: implement your test here

    def test_access(self):
        # server_session = ServerSession()
        # self.assertEqual(expected, server_session.access(*args, **kwargs))
        assert True # TODO: implement your test here

    def test_at_cmdset_get(self):
        # server_session = ServerSession()
        # self.assertEqual(expected, server_session.at_cmdset_get(**kwargs))
        assert True # TODO: implement your test here

    def test_at_disconnect(self):
        # server_session = ServerSession()
        # self.assertEqual(expected, server_session.at_disconnect())
        assert True # TODO: implement your test here

    def test_at_login(self):
        # server_session = ServerSession()
        # self.assertEqual(expected, server_session.at_login(player))
        assert True # TODO: implement your test here

    def test_at_sync(self):
        # server_session = ServerSession()
        # self.assertEqual(expected, server_session.at_sync())
        assert True # TODO: implement your test here

    def test_data_in(self):
        # server_session = ServerSession()
        # self.assertEqual(expected, server_session.data_in(text, **kwargs))
        assert True # TODO: implement your test here

    def test_data_out(self):
        # server_session = ServerSession()
        # self.assertEqual(expected, server_session.data_out(text, **kwargs))
        assert True # TODO: implement your test here

    def test_get_player(self):
        # server_session = ServerSession()
        # self.assertEqual(expected, server_session.get_player())
        assert True # TODO: implement your test here

    def test_get_puppet(self):
        # server_session = ServerSession()
        # self.assertEqual(expected, server_session.get_puppet())
        assert True # TODO: implement your test here

    def test_get_puppet_or_player(self):
        # server_session = ServerSession()
        # self.assertEqual(expected, server_session.get_puppet_or_player())
        assert True # TODO: implement your test here

    def test_log(self):
        # server_session = ServerSession()
        # self.assertEqual(expected, server_session.log(message, channel))
        assert True # TODO: implement your test here

    def test_msg(self):
        # server_session = ServerSession()
        # self.assertEqual(expected, server_session.msg(text, **kwargs))
        assert True # TODO: implement your test here

    def test_ndb_del(self):
        # server_session = ServerSession()
        # self.assertEqual(expected, server_session.ndb_del())
        assert True # TODO: implement your test here

    def test_ndb_get(self):
        # server_session = ServerSession()
        # self.assertEqual(expected, server_session.ndb_get())
        assert True # TODO: implement your test here

    def test_ndb_set(self):
        # server_session = ServerSession()
        # self.assertEqual(expected, server_session.ndb_set(value))
        assert True # TODO: implement your test here

    def test_update_session_counters(self):
        # server_session = ServerSession()
        # self.assertEqual(expected, server_session.update_session_counters(idle))
        assert True # TODO: implement your test here

if __name__ == '__main__':
    unittest.main()
